rm(list=ls())

#**** BEGIN FUNCTIONS ****#

#residuals for one variable
get_one_residual <- function(x, variables, moderators){
 numero <- as.numeric(gsub("\\D","", deparse(substitute(x)), perl=T))
 str.formula <- paste(names(variables)[numero],paste(names(moderators),collapse="+"),sep=" ~ ")
 reg <- lm(formula = as.formula(str.formula),data=cbind(variables,moderators))
 return(reg$residuals)
}

#residuals for a set of variables
get_residuals <- function(variables,moderators){
 new.data <- lapply(variables,get_one_residual,variables,moderators)
 return(as.data.frame(new.data))
}

#perform a pca with partial correlation matrix
pca_partial <- function(variables, moderators){
 new.scores <- get_residuals(variables,moderators)
 pca.object <- princomp(new.scores,cor=T)
 return(pca.object)
}

#**** END FUNCTIONS ****#

#load the dataset
library(xlsx)
setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/fact_analysis/acp_corr_partielle")
body.data <- read.xlsx(file="body_sas.xls",sheetIndex=1,header=T)

#data subdivision
body.variables <- subset(body.data, select=shoulder.girth:wrist.girth)
body.moderators <- subset(body.data, select=weight:gender)

#performing the PCA on the partial correlation matrix
body.pca <- pca_partial(body.variables,body.moderators)

#print the eigenvalues
print(body.pca$sdev^2)

#scree plot
plot(body.pca,type="lines")

#print the factor pattern for the first component
print(body.pca$loadings[,1]*body.pca$sdev[1])
